package com.bipt.intelligentapplicationorchestrationservice.enumeration;

import com.bipt.intelligentapplicationorchestrationservice.constant.MessageConstant;

/**
 * 模型生命周期状态枚举
 */
public enum ModelLifecycle {
    DESIGNING("设计中"),
    TRAINING("训练中"),
    EVALUATING("评估中"),
    DEPLOYED("已部署"),
    OFFLINE("已下线"),
    ABANDONED("已废弃");

    private final String description;

    private ModelLifecycle(String description) {
        this.description = description;
    }


    public String getDescription() {
        return description;
    }

    // 将枚举名称转换成数据库存储的字符串
    public String getDbValue() {
        return name();
    }

    // 将数据库存储的字符串转换为枚举实例
    public static ModelLifecycle getDbValueFromDb(String dbValue) {
        if(dbValue == null) return null;
        try  {
            return ModelLifecycle.valueOf(dbValue);
        } catch (IllegalArgumentException e) {
            throw new IllegalArgumentException(MessageConstant.LifeCycle_No_Exist + ":" + dbValue);
        }
    }

    // 从前端传的中文描述 -> 转换为枚举
    public static ModelLifecycle fromDescription(String description) {
        for (ModelLifecycle lifecycle : ModelLifecycle.values()) {
            if (lifecycle.getDescription().equals(description)) {
                return lifecycle;
            }
        }
        throw new IllegalArgumentException("生命周期描述非法: " + description);
    }
}